@@ -5,15 +5,15 @@ Import subpackage classes
 from andes.core.model import Model, ModelData, ModelCall  # NOQA
 
 from andes.core.param import BaseParam, DataParam, NumParam, IdxParam  # NOQA
-from andes.core.param import RefParam, TimerParam, ExtParam, RefParam  # NOQA
+from andes.core.param import TimerParam, ExtParam  # NOQA
 
 from andes.core.var import BaseVar, Algeb, State, ExtVar, ExtAlgeb, ExtState  # NOQA
 
-from andes.core.service import BaseService, ConstService, ExtService  # NOQA
-from andes.core.service import OperationService, RandomService, ReducerService, RepeaterService  # NOQA
+from andes.core.service import BaseService, ConstService, ExtService, BackRef  # NOQA
+from andes.core.service import OperationService, RandomService, NumReduce, NumRepeat, IdxRepeat  # NOQA
 
 from andes.core.discrete import Discrete, LessThan, Limiter, SortedLimiter, HardLimiter  # NOQA
-from andes.core.discrete import AntiWindupLimiter, Selector, Switcher, DeadBand  # NOQA
+from andes.core.discrete import AntiWindup, Selector, Switcher, DeadBand  # NOQA
 
 from andes.core.block import Block, Washout, Lag,  LeadLag, Piecewise  # NOQA
 from andes.core.block import LagAntiWindup, LeadLagLimit  # NOQA
