@@ -1,16 +1,10 @@
 from andes.core.var import Algeb, State
-from typing import Optional, Iterable
-from andes.core.discrete import AntiWindupLimiter
-from andes.core.service import ConstService
+from andes.core.param import dummify
+from typing import Optional, Iterable, Dict, Union
+from andes.core.discrete import Discrete, AntiWindup, LessThan
 from andes.core.triplet import JacTriplet
 
 
-class DummyValues(object):
-    def __init__(self, name):
-        self.name = name
-        self.tex_name = name
-
-
 class Block(object):
     r"""
     Base class for control blocks.
@@ -40,7 +34,7 @@ class Block(object):
 
     Warnings
     --------
-    It is suggested to have at most one level of nesting to avoid messy variable names.
+    It is a good practice to avoid more than one level of nesting, to avoid multi-underscore variable names.
 
     Examples
     --------
@@ -52,7 +46,7 @@ class Block(object):
            |
         Lag B      exports (x, y)
 
-    SomeModel instance M contains an instance of Leadlag block named A, which contains an instance of a Lag block
+    SomeModel instance M contains an instance of LeadLag block named A, which contains an instance of a Lag block
     named B. Both A and B exports two variables ``x`` and ``y``.
 
     In the code of Model, the following code is used to instantiate LeadLag ::
@@ -73,18 +67,18 @@ class Block(object):
                 self.B = Lag(u=self.y, K=self.K, T=self.T)
                 self.vars = {..., 'A': self.A}
 
-    The ``__setattr__`` magic of LeadLag takes over the construction and assigns ``B.name`` as ``A_B``,
-    given A's name provided at run time. ``self.A`` is exported with the internal name ``A`` at the end.
+    The ``__setattr__`` magic of LeadLag takes over the construction and assigns ``A_B`` to `B.name`,
+    given A's name provided at run time. `self.A` is exported with the internal name ``A`` at the end.
 
-    Again, the LeadLag instance name (``A`` in this example) MUST be provided in SomeModel's constructor for the
+    Again, the LeadLag instance name (`A` in this example) MUST be provided in `SomeModel`'s constructor for the
     name prepending to work correctly. If there is more than one level of nesting, other than the leaf-level
     block, all parent blocks' names must be provided at instantiation.
 
-    When A is picked up by ``SomeModel.__setattr__``, B is captured from A's exports. Recursively, B's variables
-    are exported, Recall that ``B.name`` is ``A_B``, following the naming rule (parent block's name + variable
+    When A is picked up by `SomeModel.__setattr__`, B is captured from A's exports. Recursively, B's variables
+    are exported, Recall that `B.name` is now ``A_B``, following the naming rule (parent block's name + variable
     name), B's internal variables become ``A_B_x`` and ``A_B_y``.
 
-    In this way, B's ``define`` needs no modification since the naming rule is the same. For example,
+    In this way, B's ``define()`` needs no modification since the naming rule is the same. For example,
     B's internal y is always ``{self.name}_y``, although B has gotten a new name ``A_B``.
 
     """
@@ -94,14 +88,14 @@ class Block(object):
         self.tex_name = tex_name if tex_name else name
         self.info = info
         self.owner = None
-        self.vars: dict = dict()
+        self.vars: Dict[str, Union[Algeb, State, Discrete]] = dict()
         self.triplets = JacTriplet()
 
     def __setattr__(self, key, value):
         # handle sub-blocks by prepending self.name
         if isinstance(value, Block):
             if self.name is None:
-                raise ValueError(f"`name` must be specified when constructing {self.class_name} instances.")
+                raise ValueError(f"Must specify `name` for {self.class_name} any instance.")
             if not value.owner:
                 value.__dict__['owner'] = self
 
@@ -229,7 +223,9 @@ class Block(object):
 
     @staticmethod
     def enforce_tex_name(fields):
-        """Enforce tex_name is not None"""
+        """
+        Enforce tex_name is not None
+        """
         if not isinstance(fields, Iterable):
             fields = [fields]
 
@@ -278,9 +274,8 @@ class PIController(Block):
         Equations implemented are
 
         .. math ::
-            \dot{x_i} = k_i * (ref - var)
-
-            y = x_i + k_i * (ref - var)
+            \dot{x_i} &= k_i * (ref - var) \\
+            y &= x_i + k_i * (ref - var)
         """
 
         self.xi.e_str = f'ki * ({self.ref.name} - {self.u.name})'
@@ -322,6 +317,68 @@ class PIControllerNumeric(Block):
         pass
 
 
+class Gain(Block):
+    r"""
+    Gain block ::
+
+        u -> K -> y
+
+    Exports an algebraic output `y`.
+    """
+    def __init__(self, u, K, name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = u
+        self.K = dummify(K)
+        self.enforce_tex_name((self.K, ))
+
+        self.y = Algeb(info='Gain output', tex_name='y')
+        self.vars = {'y': self.y}
+
+    def define(self):
+        r"""
+        Implemented equation and the initial condition are
+
+        .. math ::
+            y = K u
+            y^{(0)} = K u^{(0)}
+
+        """
+        self.y.v_str = f'{self.K.name} * {self.u.name}'
+        self.y.e_str = f'{self.K.name} * {self.u.name} - {self.name}_y'
+
+
+class Integrator(Block):
+    r"""
+    Integrator block ::
+
+        u -> K/s -> y
+
+    Exports a differential variable `y`. The initial output is specified by `y0` and default to zero.
+    """
+
+    def __init__(self, u, K, y0=0, name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = u
+        self.K = dummify(K)
+        self.enforce_tex_name((self.K, ))
+        self.y0 = y0
+
+        self.y = State(info='Integrator output', tex_name='y')
+        self.vars = {'y': self.y}
+
+    def define(self):
+        r"""
+        Implemented equation and the initial condition are
+
+        .. math ::
+            \dot{y} = K u \\
+            y^{(0)} = 0
+
+        """
+        self.y.v_str = self.y0
+        self.y.e_str = f'{self.K.name} * {self.u.name}'
+
+
 class Washout(Block):
     r"""
     Washout filter (high pass) block ::
@@ -330,30 +387,19 @@ class Washout(Block):
          u -> ------- -> y
               1 + sT
 
+    Exports state `x` (symbol `x'`) and output algebraic variable `y`.
     """
 
-    def __init__(self, u, T, K, info=None, name=None):
-        super().__init__(name=name, info=info)
-        if isinstance(T, (int, float)):
-            self.T = DummyValues(T)
-        else:
-            self.T = T
-
-        if isinstance(K, (int, float)):
-            self.K = DummyValues(K)
-        else:
-            self.K = K
-
+    def __init__(self, u, T, K, name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = u
+        self.T = dummify(T)
+        self.K = dummify(K)
         self.enforce_tex_name((self.K, self.T))
 
-        self.KT = ConstService(info='Constant K/T',
-                               tex_name=f'({self.K.tex_name}/{self.T.tex_name})',
-                               v_str=f'{self.K.name} / {self.T.name}')
-
-        self.u = u
-        self.x = State(info='State in washout filter', tex_name="x'")
-        self.y = Algeb(info='Output of washout filter', tex_name=r'y')
-        self.vars = {'KT': self.KT, 'x': self.x, 'y': self.y}
+        self.x = State(info='State in washout filter', tex_name="x'", t_const=self.T)
+        self.y = Algeb(info='Output of washout filter', tex_name=r'y', diag_eps=1e-6)
+        self.vars.update({'x': self.x, 'y': self.y})
 
     def define(self):
         r"""
@@ -362,25 +408,77 @@ class Washout(Block):
         Equations and initial values:
 
         .. math ::
-            \dot{x'} = (u - x) / T \\
-            y = u - x \\
-            x'_0 = u \\
-            y_0 = 0
+            T \dot{x'} &= (u - x) \\
+            T y &= K (u - x) \\
+            x'^{(0)} &= u \\
+            y^{(0)} &= 0
 
         """
         self.x.v_str = f'{self.u.name}'
         self.y.v_str = f'0'
 
-        self.x.e_str = f'({self.u.name} - {self.name}_x) / {self.T.name}'
-        self.y.e_str = f'{self.name}_KT * ({self.u.name} - {self.name}_x) - {self.name}_y'
+        self.x.e_str = f'({self.u.name} - {self.name}_x)'
+        self.y.e_str = f'{self.K.name} * ({self.u.name} - {self.name}_x) - {self.T.name} * {self.name}_y'
+
+
+class WashoutOrLag(Washout):
+    """
+    Washout with the capability to convert to Lag when K = 0.
+
+    Can be enabled with `zero_out`. Need to provide `name` to construct.
+
+    Exports state `x` (symbol `x'`), output algebraic variable `y`, and a LessThan block `LT`.
+
+    Parameters
+    ----------
+    zero_out : bool, optional
+        If True, ``sT`` will become 1, and the washout will become a low-pass filter.
+        If False, functions as a regular Washout.
+    """
+    def __init__(self, u, T, K, name, zero_out=False, tex_name=None, info=None):
+        super().__init__(u, T, K, name=name, tex_name=tex_name, info=info)
+        self.zero_out = zero_out
+        self.LT = LessThan(K,
+                           dummify(0),
+                           equal=True,
+                           enable=zero_out,
+                           tex_name='LT',
+                           cache=True,
+                           z0=1, z1=0)
+
+        self.vars.update({'LT': self.LT})
+
+    def define(self):
+        r"""
+        Notes
+        -----
+        Equations and initial values:
+
+        .. math ::
+            T \dot{x'} &= (u - x) \\
+            T y = z_0 K (u-x) + z_1 Tx
+            x'^{(0)} &= u \\
+            y^{(0)} &= 0
+
+        where ``z_0`` is a flag array for the greater-than-zero elements, and ``z_1`` is that for the
+        less-than or equal-to zero elements.
+        """
+
+        super().define()
+
+        self.y.v_str = f'{self.name}_LT_z0 * 0 + {self.name}_LT_z1 * {self.name}_x'
+
+        self.y.e_str = f'{self.name}_LT_z0 * {self.K.name} * ({self.u.name} - {self.name}_x) + ' \
+                       f'{self.name}_LT_z1 * {self.T.name} * {self.name}_x - ' \
+                       f'{self.T.name} * {self.name}_y'
 
 
 class Lag(Block):
     r"""
-    Lag (low pass) transfer function block ::
+    Lag (low pass filter) transfer function block ::
 
                 K
-        u -> ------ -> y
+        u -> ------ -> x
              1 + sT
 
     Exports one state variable `x` as the output.
@@ -398,14 +496,12 @@ class Lag(Block):
     def __init__(self, u, T, K, name=None, info='Lag transfer function'):
         super().__init__(name=name, info=info)
         self.u = u
-        self.T = T
-        if isinstance(K, (int, float)):
-            self.K = DummyValues(K)
-        else:
-            self.K = K
+        self.T = dummify(T)
+        self.K = dummify(K)
 
         self.enforce_tex_name((self.K, self.T))
-        self.x = State(info='State in lag transfer function', tex_name="x'")
+        self.x = State(info='State in lag transfer function', tex_name="x'",
+                       t_const=self.T)
 
         self.vars = {'x': self.x}
 
@@ -414,28 +510,30 @@ class Lag(Block):
 
         Notes
         -----
-        Equation and initial value
+        Equations and initial values are
 
         .. math ::
 
-            \dot{x'} = (u - x) / T
-
-            x'_0 = u
+            T \dot{x'} &= (Ku - x') \\
+            x'^{(0)} &= K u
 
         """
         self.x.v_str = f'{self.u.name} * {self.K.name}'
-        self.x.e_str = f'({self.K.name} * {self.u.name} - {self.name}_x) / {self.T.name}'
+        self.x.e_str = f'({self.K.name} * {self.u.name} - {self.name}_x)'
 
 
 class LagAntiWindup(Block):
     r"""
-    Lag (low pass) transfer function block with anti-windup limiter ::
+    Lag (low pass filter) transfer function block with an anti-windup limiter ::
 
-                K
-        u -> ------ -> y
-             1 + sT
+                      /-- upper --
+                     K
+             u -> ------ -> x
+                  1 + sT
+        -- lower --/
 
     Exports one state variable `x` as the output.
+    Exports one AntiWindup instance `lim`.
 
     Parameters
     ----------
@@ -450,19 +548,18 @@ class LagAntiWindup(Block):
     def __init__(self, u, T, K, lower, upper, name=None,
                  info='Lag transfer function with non-windup limiter'):
         super().__init__(name=name, info=info)
-        if isinstance(K, (int, float)):
-            self.K = DummyValues(K)
-        else:
-            self.K = K
         self.u = u
-        self.T = T
+        self.T = dummify(T)
+        self.K = dummify(K)
+
         self.lower = lower
         self.upper = upper
 
         self.enforce_tex_name((self.T, self.K))
 
-        self.x = State(info='State in lag transfer function', tex_name="x'")
-        self.lim = AntiWindupLimiter(u=self.x, lower=self.lower, upper=self.upper, tex_name='lim')
+        self.x = State(info='State in lag transfer function', tex_name="x'",
+                       t_const=self.T)
+        self.lim = AntiWindup(u=self.x, lower=self.lower, upper=self.upper, tex_name='lim')
 
         self.vars = {'x': self.x, 'lim': self.lim}
 
@@ -471,17 +568,77 @@ class LagAntiWindup(Block):
 
         Notes
         -----
-        Equation and initial value
+        Equations and initial values are
 
         .. math ::
 
-            \dot{x'} = (u - x) / T
-
-            x'_0 = u
+            T \dot{x'} &= (Ku - x) \\
+            x'^{(0)} &= K u
 
         """
         self.x.v_str = f'{self.u.name} * {self.K.name}'
-        self.x.e_str = f'{self.name}_lim_zi * ({self.K.name} * {self.u.name} - {self.name}_x) / {self.T.name}'
+        self.x.e_str = f'{self.name}_lim_zi * ({self.K.name} * {self.u.name} - {self.name}_x)'
+
+
+class Lag2ndOrd(Block):
+    r"""
+    Second order lag transfer function (low-pass filter) ::
+
+                     K
+        u -> ----------------- -> y
+             1 + sT1 + s^2 T2
+
+    Exports one two state variables (`x`, `y`), where `y` is the output.
+
+    Parameters
+    ----------
+    u
+        Input
+    K
+        Gain
+    T1
+        First order time constant
+    T2
+        Second order time constant
+    """
+
+    def __init__(self, u, K, T1, T2, name=None, tex_name=None, info=None):
+        super(Lag2ndOrd, self).__init__(name=name, tex_name=tex_name, info=info)
+
+        self.u = u
+        self.K = dummify(K)
+        self.T1 = dummify(T1)
+        self.T2 = dummify(T2)
+
+        self.enforce_tex_name((self.K, self.T1, self.T2))
+
+        self.x = State(info='State in 2nd order LPF', tex_name="x'", t_const=self.T2)
+        self.y = State(info='Output of 2nd order LPF', tex_name='y')
+
+        self.vars = {'x': self.x, 'y': self.y}
+
+    def define(self):
+        r"""
+
+        Notes
+        -------
+        Implemented equations and initial values are
+
+        .. math ::
+
+            T_2 \dot{x} &= Ku - y - T_1 x \\
+            \dot{y} &= x \\
+            x^{(0)} &= 0 \\
+            y^{(0)} &= K u
+        """
+
+        self.x.v_str = 0
+        self.x.e_str = f'{self.u.name} * {self.K.name} - ' \
+                       f'{self.name}_y - ' \
+                       f'{self.T1.name} * {self.name}_x'
+
+        self.y.v_str = f'{self.u.name} * {self.K.name}'
+        self.y.e_str = f'{self.name}_x'
 
 
 class LeadLag(Block):
@@ -492,33 +649,25 @@ class LeadLag(Block):
         u -> ------- -> y
              1 + sT2
 
-    Exports two variables: state x and output y.
+    Exports two variables: internal state `x` and output algebraic variable `y`.
 
     Parameters
     ----------
-    T1
+    T1 : BaseParam
         Time constant 1
-    T2
+    T2 : BaseParam
         Time constant 2
-
-    Notes
-    -----
-    Future implementation will allow T2 to be zero with a safe division. Any division by zero will become zero.
-
-    c = np.divide(a, b, out=np.zeros_like(a), where=b!=0)
-
     """
-    def __init__(self, u, T1, T2, name=None, info='Lead-lag transfer function', safe_div=True):
-        super().__init__(name=name, info=info)
-        self.T1 = T1
-        self.T2 = T2
+    def __init__(self, u, T1, T2, name=None, tex_name=None, info='Lead-lag transfer function'):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.T1 = dummify(T1)
+        self.T2 = dummify(T2)
         self.u = u
-        self.safe_div = safe_div  # TODO: implement me
 
         self.enforce_tex_name((self.T1, self.T2))
 
-        self.x = State(info='State in lead-lag transfer function', tex_name="x'")
-        self.y = Algeb(info='Output of lead-lag transfer function', tex_name=r'y')
+        self.x = State(info='State in lead-lag', tex_name="x'", t_const=self.T2)
+        self.y = Algeb(info='Output of lead-lag', tex_name=r'y', diag_eps=1e-6)
         self.vars = {'x': self.x, 'y': self.y}
 
     def define(self):
@@ -531,32 +680,83 @@ class LeadLag(Block):
 
         .. math ::
 
-            \dot{x'} = (u - x') / T_2 \\
-            y = \frac {T_1} {T_2} * (u - x') + x' \\
-            x'_0 = y_0 = u
+            T_2 \dot{x'} &= (u - x') \\
+            T_2  y &= T_1  (u - x') + T_2  x' \\
+            x'^{(0)} &= y^{(0)} = u
 
         """
         self.x.v_str = f'{self.u.name}'
         self.y.v_str = f'{self.u.name}'
 
-        self.x.e_str = f'({self.u.name} - {self.name}_x) / {self.T2.name}'
-        self.y.e_str = f'{self.T1.name} / {self.T2.name} * ({self.u.name} - {self.name}_x) + ' \
-                       f'{self.name}_x - ' \
-                       f'{self.name}_y'
+        self.x.e_str = f'({self.u.name} - {self.name}_x)'
+        self.y.e_str = f'{self.T1.name} * ({self.u.name} - {self.name}_x) + ' \
+                       f'{self.name}_x * {self.T2.name} - ' \
+                       f'{self.name}_y * {self.T2.name}'
+
+
+class LeadLag2ndOrd(Block):
+    r"""
+    Second-order lead-lag transfer function block ::
+
+             1 + sT3 + s^2 T4
+        u -> ----------------- -> y
+             1 + sT1 + s^2 T2
+
+    Exports two internal states (`x1` and `x2`) and output algebraic variable `y`.
+    """
+
+    def __init__(self, u, T1, T2, T3, T4, name=None, tex_name=None, info='2nd-order lead-lag'):
+        super(LeadLag2ndOrd, self).__init__(name=name, tex_name=tex_name, info=info)
+        self.u = u
+        self.T1 = dummify(T1)
+        self.T2 = dummify(T2)
+        self.T3 = dummify(T3)
+        self.T4 = dummify(T4)
+        self.enforce_tex_name((self.T1, self.T2, self.T3, self.T4))
+
+        self.x1 = State(info='State #1 in 2nd order lead-lag', tex_name="x'", t_const=self.T2)
+        self.x2 = State(info='State #2 in 2nd order lead-lag', tex_name="x''")
+        self.y = Algeb(info='Output of 2nd order lead-lag', tex_name='y', diag_eps=1e-6)
+
+        self.vars = {'x1': self.x1, 'x2': self.x2, 'y': self.y}
+
+    def define(self):
+        r"""
+        Notes
+        -----
+        Implemented equations and initial values are
+
+        .. math ::
+            T_2 \dot{x}_1 &= u - x_2 - T_1 x_1 \\
+            \dot{x}_2 &= x_1 \\
+            T_2 y &= T_2 x_2 + T_2 T_3 x_1 + T_4 (u - x_2 - T_1 x_1) \\
+            x_1^{(0)} &= 0 \\
+            x_2^{(0)} &= y^{(0)} = u
+
+        """
+        self.x1.e_str = f'{self.u.name} - {self.name}_x2 - {self.T1.name} * {self.name}_x1'
+        self.x2.e_str = f'{self.name}_x1'
+        self.y.e_str = f'{self.T2.name} * {self.name}_x2 + ' \
+                       f'{self.T2.name} * {self.T3.name} * {self.name}_x1 + ' \
+                       f'{self.T4.name} * ({self.u.name} - {self.name}_x2 - {self.T1.name} * {self.name}_x1) - ' \
+                       f'{self.T2.name} * {self.name}_y'
+
+        self.x1.v_str = 0
+        self.x2.v_str = f'{self.u.name}'
+        self.y.v_str = f'{self.u.name}'
 
 
 class LeadLagLimit(Block):
     r"""
     Lead-Lag transfer function block with hard limiter (series implementation) ::
 
-                       ___upper___
-                      /
-                  1 + sT1
-           u ->   -------  -> y
-                  1 + sT2
-        __lower____/
+                                ___upper___
+              1 + sT1          /
+        u ->  -------  -> ynl / -> y
+              1 + sT2        /
+                  __lower___/
 
-    Exports four variables: state `x`, output before hard limiter `ynl`, output `y`, and limiter `lim`,
+    Exports four variables: state `x`, output before hard limiter `ynl`, output `y`, and AntiWindup `lim`.
 
     """
     def __init__(self, u, T1, T2, lower, upper, name=None, info='Lead-lag transfer function'):
@@ -568,10 +768,11 @@ class LeadLagLimit(Block):
         self.upper = upper
         self.enforce_tex_name((self.T1, self.T2))
 
-        self.x = State(info='State in lead-lag transfer function', tex_name="x'")
+        self.x = State(info='State in lead-lag transfer function', tex_name="x'", t_const=self.T2)
         self.ynl = Algeb(info='Output of lead-lag transfer function before limiter', tex_name=r'y_{nl}')
-        self.y = Algeb(info='Output of lead-lag transfer function after limiter', tex_name=r'y')
-        self.lim = AntiWindupLimiter(u=self.ynl, lower=self.lower, upper=self.upper)
+        self.y = Algeb(info='Output of lead-lag transfer function after limiter', tex_name=r'y',
+                       diag_eps=1e-6)
+        self.lim = AntiWindup(u=self.ynl, lower=self.lower, upper=self.upper)
 
         self.vars = {'x': self.x, 'ynl': self.ynl, 'y': self.y, 'lim': self.lim}
 
@@ -581,23 +782,23 @@ class LeadLagLimit(Block):
         Notes
         -----
 
-        Implemented equations and initial values
+        Implemented control block equations (without limiter) and initial values
 
         .. math ::
 
-            \dot{x'} = (u - x') / T_2 \\
-            y = \frac {T_1} {T_2} * (u - x') + x' \\
-            x'_0 = y_0 = u
+            T_2 \dot{x'} &= (u - x') \\
+            T_2 y &= T_1  (u - x') + T_2  x' \\
+            x'^{(0)} &= y^{(0)} = u
 
         """
         self.x.v_str = f'{self.u.name}'
         self.ynl.v_str = f'{self.u.name}'
         self.y.v_str = f'{self.u.name}'
 
-        self.x.e_str = f'({self.u.name} - {self.name}_x) / {self.T2.name}'
-        self.ynl.e_str = f'{self.T1.name} / {self.T2.name} * ({self.u.name} - {self.name}_x) + ' \
-                         f'{self.name}_x - ' \
-                         f'{self.name}_ynl'
+        self.x.e_str = f'({self.u.name} - {self.name}_x)'
+        self.ynl.e_str = f'{self.T1.name} * ({self.u.name} - {self.name}_x) + ' \
+                         f'{self.name}_x * {self.T2.name} - ' \
+                         f'{self.name}_ynl * {self.T2.name}'
 
         self.y.e_str = f'{self.name}_ynl * {self.name}_lim_zi + ' \
                        f'{self.lower.name} * {self.name}_lim_zl + ' \
