@@ -1,6 +1,6 @@
 from typing import Optional, Union, List
-
-from andes.core.param import BaseParam
+from andes.core.param import BaseParam, DummyValue
+from andes.core.service import BaseService
 from andes.models.group import GroupBase
 from andes.shared import np, ndarray
 
@@ -40,7 +40,7 @@ class BaseVar(object):
                  tex_name: Optional[str] = None,
                  info: Optional[str] = None,
                  unit: Optional[str] = None,
-                 v_str: Optional[str] = None,
+                 v_str: Optional[Union[str, float]] = None,
                  v_iter: Optional[str] = None,
                  e_str: Optional[str] = None,
                  v_setter: Optional[bool] = False,
@@ -48,7 +48,6 @@ class BaseVar(object):
                  addressable: Optional[bool] = True,
                  export: Optional[bool] = True,
                  diag_eps: Optional[float] = 0.0,
-                 **kwargs
                  ):
 
         self.name = name
@@ -72,7 +71,7 @@ class BaseVar(object):
         self.e_setter = e_setter  # True if this var sets the equation value
         self.addressable = addressable  # True if this var needs to be assigned an address FIXME: not in use
         self.export = export  # True if this var's value needs to exported
-        self.diag_eps = diag_eps  # small value to be added to the jacobian matrix
+        self.diag_eps = diag_eps  # small value to be added to the Jacobian matrix
 
     def reset(self):
         self.a = np.array([], dtype=int)
@@ -152,9 +151,10 @@ class State(BaseVar):
                  tex_name: Optional[str] = None,
                  info: Optional[str] = None,
                  unit: Optional[str] = None,
-                 v_str: Optional[str] = None,
+                 v_str: Optional[Union[str, float]] = None,
                  v_iter: Optional[str] = None,
                  e_str: Optional[str] = None,
+                 t_const: Optional[Union[BaseParam, DummyValue]] = None,
                  v_setter: Optional[bool] = False,
                  e_setter: Optional[bool] = False,
                  addressable: Optional[bool] = True,
@@ -174,6 +174,7 @@ class State(BaseVar):
                          export=export,
                          diag_eps=diag_eps,
                          )
+        self.t_const = t_const
 
 
 class ExtVar(BaseVar):
@@ -190,7 +191,7 @@ class ExtVar(BaseVar):
         Name of the source model
     src : str
         Source variable name
-    indexer : BaseParam
+    indexer : BaseParam, BaseService
         A parameter of the hosting model, used as indices into
         the source model and variable. If is None, the source
         variable address will be fully copied.
@@ -210,11 +211,33 @@ class ExtVar(BaseVar):
     def __init__(self,
                  model: str,
                  src: str,
-                 indexer: Optional[Union[List, ndarray, BaseParam]] = None,
-                 *args,
-                 **kwargs):
-        super().__init__(*args, **kwargs)
-        self.initialized = False
+                 indexer: Optional[Union[List, ndarray, BaseParam, BaseService]] = None,
+                 name: Optional[str] = None,
+                 tex_name: Optional[str] = None,
+                 info: Optional[str] = None,
+                 unit: Optional[str] = None,
+                 v_str: Optional[Union[str, float]] = None,
+                 v_iter: Optional[str] = None,
+                 e_str: Optional[str] = None,
+                 v_setter: Optional[bool] = False,
+                 e_setter: Optional[bool] = False,
+                 addressable: Optional[bool] = True,
+                 export: Optional[bool] = True,
+                 diag_eps: Optional[float] = 0.0,
+                 ):
+        super().__init__(name=name,
+                         tex_name=tex_name,
+                         info=info,
+                         unit=unit,
+                         v_str=v_str,
+                         v_iter=v_iter,
+                         e_str=e_str,
+                         v_setter=v_setter,
+                         e_setter=e_setter,
+                         addressable=addressable,
+                         export=export,
+                         diag_eps=diag_eps,
+                         )
         self.model = model
         self.src = src
         self.indexer = indexer
@@ -297,6 +320,42 @@ class ExtState(ExtVar):
     e_code = 'f'
     v_code = 'x'
 
+    def __init__(self,
+                 model: str,
+                 src: str,
+                 indexer: Optional[Union[List, ndarray, BaseParam, BaseService]] = None,
+                 name: Optional[str] = None,
+                 tex_name: Optional[str] = None,
+                 info: Optional[str] = None,
+                 unit: Optional[str] = None,
+                 v_str: Optional[str] = None,
+                 v_iter: Optional[str] = None,
+                 e_str: Optional[str] = None,
+                 t_const: Optional[Union[BaseParam, DummyValue]] = None,
+                 v_setter: Optional[bool] = False,
+                 e_setter: Optional[bool] = False,
+                 addressable: Optional[bool] = True,
+                 export: Optional[bool] = True,
+                 diag_eps: Optional[float] = 0.0,
+                 ):
+        super().__init__(model=model,
+                         src=src,
+                         indexer=indexer,
+                         name=name,
+                         tex_name=tex_name,
+                         info=info,
+                         unit=unit,
+                         v_str=v_str,
+                         v_iter=v_iter,
+                         e_str=e_str,
+                         v_setter=v_setter,
+                         e_setter=e_setter,
+                         addressable=addressable,
+                         export=export,
+                         diag_eps=diag_eps,
+                         )
+        self.t_const = t_const
+
 
 class ExtAlgeb(ExtVar):
     e_code = 'g'
