@@ -12,7 +12,9 @@ non_jit = OrderedDict([
     ('synchronous', ['GENCLS', 'GENROU']),
     ('governor', ['TG2', 'TGOV1']),
     ('exciter', ['EXDC2']),
+    ('pss', ['IEEEST']),
     ('measurement', ['BusFreq', 'BusROCOF']),
+    ('coi', ['COI', ]),
     ('dcbase', ['Node', 'Ground', 'R', 'L', 'C', 'RCp', 'RCs', 'RLs', 'RLCs', 'RLCp']),
     ('vsc', ['VSCShunt']),
     ('timer', ['Toggler', 'Fault']),
