@@ -1,7 +1,7 @@
-from andes.core.param import RefParam, ExtParam
+from andes.core.param import ExtParam
 from andes.core.model import Model, ModelData
 from andes.core.var import ExtAlgeb, Algeb  # NOQA
-from andes.core.service import ReducerService, RepeaterService
+from andes.core.service import NumReduce, NumRepeat, BackRef
 from andes.shared import np
 
 
@@ -18,13 +18,13 @@ class Area(AreaData, Model):
         self.flags.update({'pflow': True,
                            'tds': True})
 
-        self.Bus = RefParam(export=False)
-        self.ACTopology = RefParam(export=False)
+        self.Bus = BackRef()
+        self.ACTopology = BackRef()
 
         # --------------------Experiment Zone--------------------
         self.Vn = ExtParam(model='Bus', src='Vn', indexer=self.ACTopology, export=False)
-        self.Vn_sum = ReducerService(u=self.Vn, fun=np.sum, ref=self.Bus)
-        self.Vn_sum_rep = RepeaterService(u=self.Vn_sum, ref=self.Bus)
+        self.Vn_sum = NumReduce(u=self.Vn, fun=np.sum, ref=self.Bus)
+        self.Vn_sum_rep = NumRepeat(u=self.Vn_sum, ref=self.Bus)
 
         self.a = ExtAlgeb(model='ACTopology', src='a', indexer=self.ACTopology,
                           info='Bus voltage angle')
