@@ -1,5 +1,5 @@
 """
-Andes plotting tool
+The Andes plotting tool.
 """
 
 import logging
@@ -18,17 +18,17 @@ logger = logging.getLogger(__name__)
 
 class TDSData(object):
     """
-    A time-domain simulation data container for loading, extracing and
-    plotting data
+    A data container for loading and plotting results from Andes time-domain simulation.
     """
 
-    def __init__(self, file_name_full=None, mode='file', dae=None, path=None):
+    def __init__(self, full_name=None, mode='file', dae=None, path=None):
         # paths and file names
         self._mode = mode
-        self.file_name_full = file_name_full
+        self.full_name = full_name
         self.dae = dae
         self._path = path if path else os.getcwd()
         self.file_name = None
+        self.file_ext = None
         self._npy_file = None
         self._lst_file = None
 
@@ -54,7 +54,7 @@ class TDSData(object):
         self._latex_warn = True
 
     def _process_names(self):
-        self.file_name, _ = os.path.splitext(self.file_name_full)
+        self.file_name, self.file_ext = os.path.splitext(self.full_name)
         self._npy_file = os.path.join(self._path, self.file_name + '.npy')
         self._lst_file = os.path.join(self._path, self.file_name + '.lst')
         self._csv_file = os.path.join(self._path, self.file_name + '.csv')
@@ -67,7 +67,7 @@ class TDSData(object):
 
         self._idx = list(range(self.nvars))
         self._uname = ['Time [s]'] + dae.x_name + dae.y_name + dae.z_name
-        self._fname = ['$Time [s]$'] + dae.x_tex_name + dae.y_tex_name + dae.z_tex_name
+        self._fname = ['Time [s]'] + dae.x_tex_name + dae.y_tex_name + dae.z_tex_name
         self._data = dae.ts.txyz
 
         self.file_name = dae.system.files.name
@@ -75,12 +75,11 @@ class TDSData(object):
     def load_lst(self):
         """
         Load the lst file into internal data structures `_idx`, `_fname`, `_uname`, and counts the number of
-        variables to `nvars`
+        variables to `nvars`.
 
         Returns
         -------
         None
-
         """
         with open(self._lst_file, 'r') as fd:
             lines = fd.readlines()
@@ -104,12 +103,12 @@ class TDSData(object):
 
     def find(self, query, exclude=None, formatted=False, idx_only=False):
         """
-        Return variable names and indices matching `query`
+        Return variable names and indices matching `query`.
 
         Parameters
         ----------
         query : str
-            The string for querying variables
+            The string for querying variables. Multiple conditions can be separated by comma without space.
         exclude  : str, optional
             A string pattern to be excluded
         formatted : bool, optional
@@ -128,13 +127,15 @@ class TDSData(object):
 
         found_idx, found_names = list(), list()
 
+        query_list = query.split(',')
         for idx, name in zip(self._idx, names):
-            if re.search(query, name):
-                if exclude and re.search(exclude, name):
-                    continue
+            for q in query_list:
+                if re.search(q, name):
+                    if exclude and re.search(exclude, name):
+                        continue
 
-                found_idx.append(idx)
-                found_names.append(name)
+                    found_idx.append(idx)
+                    found_names.append(name)
 
         if idx_only:
             return found_idx
@@ -143,7 +144,7 @@ class TDSData(object):
 
     def load_npy_or_csv(self, delimiter=','):
         """
-        Load the npy or csv file into internal data structures `self._data`
+        Load the npy or csv file into internal data structures `self._data`.
 
         Parameters
         ----------
@@ -163,7 +164,7 @@ class TDSData(object):
 
     def get_values(self, idx):
         """
-        Return the variable values at the given indices
+        Return the variable values at the given indices.
 
         Parameters
         ----------
@@ -179,7 +180,7 @@ class TDSData(object):
 
     def get_header(self, idx, formatted=False):
         """
-        Return a list of the variable names at the given indices
+        Return a list of the variable names at the given indices.
 
         Parameters
         ----------
@@ -203,7 +204,7 @@ class TDSData(object):
     def export_csv(self, path=None, idx=None, header=None, formatted=False,
                    sort_idx=True, fmt='%.18e'):
         """
-        Export to a csv file
+        Export to a csv file.
 
         Parameters
         ----------
@@ -242,8 +243,10 @@ class TDSData(object):
 
     def plot(self, yidx, xidx=(0,), a=None, ycalc=None,
              left=None, right=None, ymin=None, ymax=None, ytimes=None,
-             xlabel=None, ylabel=None, legend=True, grid=False,
-             latex=True, dpi=150, savefig=None, show=True, use_bqplot=False, **kwargs):
+             xlabel=None, ylabel=None,
+             legend=True, grid=False, greyscale=False,
+             latex=True, dpi=150, savefig=None, save_format=None, show=True,
+             use_bqplot=False, latex_warn=True, **kwargs):
         """
         Entery function for plot scripting. This function retrieves the x and y values based
         on the `xidx` and `yidx` inputs and then calls `plot_data()` to do the actual plotting.
@@ -279,11 +282,12 @@ class TDSData(object):
         x_value = self.get_values(xidx)
         y_value = self.get_values(yidx)
 
+        # header: names for variables
+        # axis labels: the texts next to axes
         x_header = self.get_header(xidx, formatted=latex)
         y_header = self.get_header(yidx, formatted=latex)
 
         ytimes = float(ytimes) if ytimes is not None else ytimes
-
         if ytimes and (ytimes != 1):
             y_scale_func = scale_func(ytimes)
         else:
@@ -301,14 +305,18 @@ class TDSData(object):
 
             return self.bqplot_data(xdata=x_value, ydata=y_value, xheader=x_header, yheader=y_header,
                                     left=left, right=right, ymin=ymin, ymax=ymax,
-                                    xlabel=xlabel, ylabel=ylabel, legend=legend, grid=grid,
-                                    latex=latex, dpi=dpi, savefig=savefig, show=show, **kwargs)
+                                    xlabel=xlabel, ylabel=ylabel, legend=legend, grid=grid, greyscale=greyscale,
+                                    latex=latex, dpi=dpi, savefig=savefig, save_format=save_format, show=show,
+                                    latex_warn=latex_warn,
+                                    **kwargs)
 
         else:
             return self.plot_data(xdata=x_value, ydata=y_value, xheader=x_header, yheader=y_header,
                                   left=left, right=right, ymin=ymin, ymax=ymax,
-                                  xlabel=xlabel, ylabel=ylabel, legend=legend, grid=grid,
-                                  latex=latex, dpi=dpi, savefig=savefig, show=show, **kwargs)
+                                  xlabel=xlabel, ylabel=ylabel, legend=legend, grid=grid, greyscale=greyscale,
+                                  latex=latex, dpi=dpi, savefig=savefig, save_format=save_format, show=show,
+                                  latex_warn=latex_warn,
+                                  **kwargs)
 
     def data_to_df(self):
         """Convert to pandas.DataFrame"""
@@ -322,7 +330,7 @@ class TDSData(object):
 
     def bqplot_data(self, xdata, ydata, xheader=None, yheader=None, xlabel=None, ylabel=None,
                     left=None, right=None, ymin=None, ymax=None, legend=True, grid=False, fig=None,
-                    latex=True, dpi=150, greyscale=False, savefig=None, show=True, **kwargs):
+                    latex=True, dpi=150, greyscale=False, savefig=None, save_format=None, show=True, **kwargs):
         """
         Plot with ``bqplot``. Experimental and imcomplete.
         """
@@ -346,7 +354,8 @@ class TDSData(object):
 
     def plot_data(self, xdata, ydata, xheader=None, yheader=None, xlabel=None, ylabel=None, line_styles=None,
                   left=None, right=None, ymin=None, ymax=None, legend=True, grid=False, fig=None, ax=None,
-                  latex=True, dpi=100, greyscale=False, savefig=None, show=True, **kwargs):
+                  latex=True, dpi=150, greyscale=False, savefig=None, save_format=None, show=True,
+                  latex_warn=True, **kwargs):
         """
         Plot lines for the supplied data and options. This functions takes `xdata` and `ydata` values. If
         you provide variable indices instead of values, use `plot()`.
@@ -367,10 +376,10 @@ class TDSData(object):
             A list containing the variable names for the y-axis variable
 
         xlabel : str
-            A label for the x axis
+            Text label for the x axis
 
         ylabel : str
-            A label for the y axis
+            Text label for the y axis
 
         left : float
             The starting value of the x axis
@@ -393,16 +402,16 @@ class TDSData(object):
             Matplotlib axis object to draw the lines on
         latex : bool
             True to enable latex and False to disable
-
-        dpi : int
-            Dots per inch for screen print or save
         greyscale : bool
             True to use greyscale, False otherwise
         savefig : bool
             True to save to png figure file
+        save_format : str
+            File extension string (pdf, png or jpg) for the savefig format
+        dpi : int
+            Dots per inch for screen print or save. savefig uses a minimum of 200 dpi
         show : bool
             True to show the image
-
         kwargs
             Optional kwargs
 
@@ -411,22 +420,20 @@ class TDSData(object):
         (fig, ax)
             The figure and axis handles
         """
+        mpl.rc('font', family='Arial', size=12)
+        self._latex_warn = latex_warn
 
         if not isinstance(ydata, np.ndarray):
-            TypeError("ydata must be numpy array. Retrieve with get_values().")
+            TypeError("ydata must be a numpy array. Retrieve with get_values().")
 
         if ydata.ndim == 1:
             ydata = ydata.reshape((-1, 1))
 
         n_lines = ydata.shape[1]
 
-        mpl.rc('font', family='Arial', size=12)
-
         using_latex = set_latex(latex)
-
         if using_latex and self._latex_warn:
-            logger.info('Using LaTeX for rendering.')
-            logger.info('If the rendering takes too long or an error occurs:')
+            logger.info('Using LaTeX for rendering. If an error occurs:')
             logger.info('a) If you are using `andes plot`, disable with optino "-d",')
             logger.info('b) If you are using `plot()`, set "latex=False".')
             self._latex_warn = False
@@ -459,17 +466,13 @@ class TDSData(object):
                     color='0.2' if greyscale else None,
                     )
 
-        if xlabel:
-            if using_latex:
-                ax.set_xlabel(label_texify(xlabel))
+        if xlabel is not None:
+            ax.set_xlabel(xlabel)
         else:
             ax.set_xlabel(xheader[0])
 
         if ylabel:
-            if using_latex:
-                ax.set_ylabel(label_texify(ylabel))
-            else:
-                ax.set_ylabel(ylabel)
+            ax.set_ylabel(ylabel)
 
         ax.ticklabel_format(useOffset=False)
 
@@ -489,20 +492,23 @@ class TDSData(object):
         plt.draw()
 
         if savefig:
-            count = 1
+            if save_format is None:
+                save_format = 'png'
 
+            if dpi is None:
+                dpi = 200
+            else:
+                dpi = max(dpi, 200)
+
+            count = 1
             while True:
-                outfile = self.file_name + '_' + str(count) + '.png'
+                outfile = f'{self.file_name}_{count}.{save_format}'
                 if not os.path.isfile(outfile):
                     break
                 count += 1
 
-            try:
-                fig.savefig(outfile, dpi=dpi)
-                logger.info(f'Figure saved to <{outfile}>')
-            except IOError:
-                logger.error('* An unknown error occurred. Try disabling LaTeX with "-d".')
-                return
+            fig.savefig(outfile, dpi=dpi)
+            logger.info(f'Figure saved to "{outfile}".')
 
         if show:
             plt.show()
@@ -625,8 +631,12 @@ def eig_plot(name, args):
         pass
 
 
-def tdsplot(filename, y, x=(0,), tocsv=False,
-            find=None, xargs=None, exclude=None, **kwargs):
+def tdsplot(filename, y, x=(0,),
+            tocsv=False,
+            find=None,
+            xargs=None,
+            exclude=None,
+            **kwargs):
     """
     TDS plot main function based on the new TDSData class
 
@@ -678,13 +688,20 @@ def tdsplot(filename, y, x=(0,), tocsv=False,
 
 
 def check_init(yval, yl):
-    """"Check initialization by comparing t=0 and t=end values"""
+    """"
+    Check initialization by comparing t=0 and t=end values for a flat run.
+
+    Warnings
+    --------
+    This function is deprecated as the initialization check feature is built into TDS.
+    See ``TDS.test_initialization()``.
+    """
     suspect = []
     for var, label in zip(yval, yl):
         if abs(var[0] - var[-1]) >= 1e-6:
             suspect.append(label)
     if suspect:
-        logger.error('Initialization failure:')
+        logger.error('Initialization failed:')
         logger.error(', '.join(suspect))
     else:
         logger.error('Initialization is correct.')
@@ -722,7 +739,7 @@ def scale_func(k):
     return lambda y_values_input: k * y_values_input
 
 
-def label_texify(label):
+def label_latexify(label):
     """
     Convert a label to latex format by appending surrounding $ and escaping spaces
 
@@ -741,7 +758,7 @@ def label_texify(label):
 
 def set_latex(enable=True):
     """
-    Enables latex for matplotlib based on the `with_latex` option and `dvipng` availability
+    Enables LaTeX for matplotlib based on the `with_latex` option and `dvipng` availability.
 
     Parameters
     ----------
@@ -751,11 +768,10 @@ def set_latex(enable=True):
     Returns
     -------
     bool
-        True for latex on and False for off
+        True for LaTeX on, False for off
     """
 
     has_dvipng = find_executable('dvipng')
-
     if has_dvipng and enable:
         mpl.rc('text', usetex=True)
         return True
