@@ -21,11 +21,29 @@ class TDS(BaseRoutine):
         self.config.add(OrderedDict((('tol', 1e-4),
                                      ('t0', 0.0),
                                      ('tf', 20.0),
-                                     ('fixt', 1),       # use fixed time step
-                                     ('shrinkt', 1),    # shrink step size if `max_iter` is reached
-                                     ('tstep', 1/30),   # fixed step size / initial step size
+                                     ('fixt', 1),
+                                     ('shrinkt', 1),
+                                     ('tstep', 1/30),
                                      ('max_iter', 15),
                                      )))
+        self.config.add_extra("_help",
+                              tol="convergence tolerance",
+                              t0="simulation starting time",
+                              tf="simulation ending time",
+                              fixt="use fixed step size (1) or variable (0)",
+                              shrinkt='shrink step size for fixed method if not converged',
+                              tstep='the initial step step size',
+                              max_iter='maximum number of iterations',
+                              )
+        self.config.add_extra("_alt",
+                              tol="float",
+                              t0=">=0",
+                              tf=">t0",
+                              fixt=(0, 1),
+                              shrinkt=(0, 1),
+                              tstep='float',
+                              max_iter='>=10',
+                              )
         # overwrite `tf` from command line
         if system.options.get('tf') is not None:
             self.config.tf = system.options.get('tf')
@@ -39,6 +57,8 @@ class TDS(BaseRoutine):
         self.deltatmax = 0
         self.h = 0
         self.next_pc = 0
+        self.eye = None
+        self.Teye = None
 
         # internal status
         self.converged = False
@@ -52,7 +72,7 @@ class TDS(BaseRoutine):
         self.plotter = None
         self.initialized = False
 
-    def _initialize(self):
+    def init(self):
         """
         Initialize the status, storage and values for TDS.
 
@@ -65,6 +85,7 @@ class TDS(BaseRoutine):
         t0, _ = elapsed()
         system = self.system
         self._reset()
+        self._load_pert()
         system.set_address(models=self.tds_models)
         system.set_dae_names(models=self.tds_models)
 
@@ -73,11 +94,12 @@ class TDS(BaseRoutine):
         system.store_sparse_pattern(models=self.pflow_tds_models)
         system.store_adder_setter(models=self.pflow_tds_models)
         system.vars_to_models()
-        system.initialize(self.tds_models)
+        system.init(self.tds_models)
         system.store_switch_times(self.tds_models)
-        self.initialized = self.test_initialization()
+        self.eye = spdiag([1] * system.dae.n)
+        self.Teye = spdiag(system.dae.Tf.tolist()) * self.eye
 
-        self._load_pert()
+        self.initialized = self.test_initialization()
         _, s1 = elapsed(t0)
 
         if self.initialized is True:
@@ -101,11 +123,14 @@ class TDS(BaseRoutine):
         out.append('')
         out.append('-> Time Domain Simulation Summary:')
         out.append(f'Sparse Solver: {self.solver.sparselib.upper()}')
-        out.append(f'Simulation time: {self.config.t0}-{self.config.tf}s')
+        out.append(f'Simulation time: {self.config.t0}-{self.config.tf}sec.')
         if self.config.fixt == 1:
-            out.append(f'Fixed step size: h={self.config.tstep:.4g}s')
+            msg = f'Fixed step size: h={1000 * self.config.tstep:.4g}msec.'
+            if self.config.shrinkt == 1:
+                msg += ', shrink if not converged'
+            out.append(msg)
         else:
-            out.append(f'Variable step size: h0={self.config.tstep:.4g}s')
+            out.append(f'Variable step size: h0={1000 * self.config.tstep:.4g}msec.')
 
         out_str = '\n'.join(out)
         logger.info(out_str)
@@ -129,7 +154,7 @@ class TDS(BaseRoutine):
             return ret
 
         self.summary()
-        self._initialize()
+        self.init()
         self.pbar = tqdm(total=100, ncols=70, unit='%', file=sys.stdout, disable=disable_pbar)
 
         t0, _ = elapsed()
@@ -137,7 +162,7 @@ class TDS(BaseRoutine):
             if self.calc_h() == 0:
                 self.pbar.close()
                 logger.error(f"Simulation terminated at t={system.dae.t:.4f}.")
-                ret = False
+                ret = False   # FIXME: overwritten
                 break
 
             if self.callpert is not None:
@@ -174,6 +199,12 @@ class TDS(BaseRoutine):
 
         return ret
 
+    def rewind(self, t):
+        """
+        TODO: rewind to a past time.
+        """
+        pass
+
     def load_plotter(self):
         """
         Manually load a plotter into ``TDS.plotter``.
@@ -243,34 +274,33 @@ class TDS(BaseRoutine):
                 system.j_update(models=self.pflow_tds_models)
                 self.solver.factorize = True
 
-            # solve trapezoidal rule integration
-            In = spdiag([1] * dae.n)
-            self.Ac = sparse([[In - self.h * 0.5 * dae.fx, dae.gx],
+            # solve implicit trapezoidal method (ITM) integration
+            self.Ac = sparse([[self.Teye - self.h * 0.5 * dae.fx, dae.gx],
                               [-self.h * 0.5 * dae.fy, dae.gy]], 'd')
-            # reset q as well
-            q = dae.x - self.x0 - self.h * 0.5 * (dae.f + self.f0)
+            # equation `q = 0` is the implicit form of differential equations using ITM
+            q = dae.Tf * (dae.x - self.x0) - self.h * 0.5 * (dae.f + self.f0)
+
+            # reset the corresponding q elements for pegged anti-windup limiter
             for item in system.antiwindups:
                 if len(item.x_set) > 0:
                     for key, val in item.x_set:
                         np.put(q, key[np.where(item.zi == 0)], 0)
-
             qg = np.hstack((q, dae.g))
 
             inc = self.solver.solve(self.Ac, -matrix(qg))
 
             # check for np.nan first
             if np.isnan(inc).any():
-                self.pbar.close()
                 logger.error(f'NaN found in solution. Convergence not likely')
                 self.niter = self.config.max_iter + 1
                 self.busted = True
                 break
 
-            # reset really small values to avoid anti-windup limiter flag jumps
-            inc[np.where(np.abs(inc) < 1e-12)] = 0
+            inc[np.where(np.abs(inc) < 1e-12)] = 0  # reset really small values to reduce limiter chattering
             # set new values
             dae.x += np.ravel(np.array(inc[:dae.n]))
             dae.y += np.ravel(np.array(inc[dae.n: dae.n + dae.m]))
+
             system.vars_to_models()
 
             # calculate correction
@@ -395,7 +425,7 @@ class TDS(BaseRoutine):
             As = system.dae.fx - system.dae.fy * B
             freq = max(abs(As[0, 0]), 1)
         else:
-            freq = 20.0
+            freq = 30.0
 
         if freq > system.config.freq:
             freq = float(system.config.freq)
@@ -403,7 +433,7 @@ class TDS(BaseRoutine):
         tspan = abs(config.tf - config.t0)
         tcycle = 1 / freq
 
-        self.deltatmax = min(3 * tcycle, tspan / 100.0)
+        self.deltatmax = min(tcycle, tspan / 100.0)
         self.deltat = min(tcycle, tspan / 100.0)
         self.deltatmin = min(tcycle / 500, self.deltatmax / 20)
 
@@ -456,6 +486,8 @@ class TDS(BaseRoutine):
         self.deltatmax = 0
         self.h = 0
         self.next_pc = 0.1
+        self.eye = None
+        self.Teye = None
 
         self.converged = False
         self.busted = False
