@@ -14,8 +14,8 @@ from andes.models import non_jit
 from andes.variables import FileMan, DAE
 from andes.routines import all_routines
 from andes.utils.tab import Tab
-from andes.utils.paths import get_config_path, get_pkl_path
-from andes.core import Config, BaseParam, Model, ExtVar, AntiWindupLimiter
+from andes.utils.paths import get_config_path, get_pkl_path, confirm_overwrite
+from andes.core import Config, BaseParam, Model, ExtVar, AntiWindup
 
 from andes.shared import np, spmatrix, jac_names
 logger = logging.getLogger(__name__)
@@ -28,11 +28,34 @@ else:
 
 class System(object):
     """
-    The ANDES power system class.
+    System contains models and routines for modeling and simulation.
 
-    This class stores model and routine instances as attributes.
+    System contains a several special `OrderedDict` member attributes for housekeeping.
+    These attributes include `models`, `groups`, `routines` and `calls` for loaded models, groups,
+    analysis routines, and generated numerical function calls, respectively.
+
+    Notes
+    -----
+    System stores model and routine instances as attributes.
     Model and routine attribute names are the same as their class names.
-    For example, the ``Bus`` instance of ``system`` is accessible at ``system.Bus``.
+    For example, `Bus` is stored at ``system.Bus``, the power flow calculation routine is at
+    ``system.PFlow``, and the numerical DAE instance is at ``system.dae``. See attributes for the list of
+    attributes.
+
+    Attributes
+    ----------
+    dae : andes.variables.dae.DAE
+        Numerical DAE storage
+    files : andes.variables.fileman.FileMan
+        File path storage
+    config : andes.core.config.Config
+        System config storage
+    models : OrderedDict
+        model name and instance pairs
+    groups : OrderedDict
+        group name and instance pairs
+    routines : OrderedDict
+        routine name and instance pairs
     """
     def __init__(self,
                  case: Optional[str] = None,
@@ -68,15 +91,27 @@ class System(object):
                                      ('ipadd', 1),
                                      )))
 
-        # file path manager
-        self.files = FileMan(case=case, **self.options)
-        # numerical DAE storage
-        self.dae = DAE(system=self)
+        self.config.add_extra("_help",
+                              freq='base frequency [Hz]',
+                              mva='system base MVA',
+                              store_z='store limiter status in TDS output',
+                              ipadd='Use spmatrix.ipadd if available',
+                              )
+        self.config.add_extra("_alt",
+                              freq="float",
+                              mva="float",
+                              store_z=(0, 1),
+                              ipadd=(0, 1),
+                              )
+        self.config.check()
+
+        self.files = FileMan(case=case, **self.options)    # file path manager
+        self.dae = DAE(system=self)                        # numerical DAE storage
 
         # dynamic imports of groups, models and routines
-        self._group_import()
-        self._model_import()
-        self._routine_import()  # routine imports come after models
+        self.import_groups()
+        self.import_models()
+        self.import_routines()  # routine imports come after models
 
         self._models_flag = {'pflow': self.find_models('pflow'),
                              'tds': self.find_models('tds'),
@@ -96,9 +131,33 @@ class System(object):
 
     def prepare(self, quick=False):
         """
-        Prepare classes and lambda functions
+        Generate numerical functions from symbolically defined models.
 
-        Anything in this function should be independent of test case
+        All procedures in this function must be independent of test case.
+
+        Parameters
+        ----------
+        quick : bool, optional
+            True to skip pretty-print generation to reduce code generation time.
+
+        Examples
+        --------
+        If one needs to print out LaTeX-formatted equations in a Jupyter Notebook, one need to generate such
+        equations with ::
+
+            import andes
+            sys = andes.prepare()
+
+        Alternatively, one can explicitly create a System and generate the code ::
+
+            import andes
+            sys = andes.System()
+            sys.prepare()
+
+        Warnings
+        --------
+        Generated lambda functions will be serialized to file, but pretty prints (SymPy objects) can only exist in
+        the System instance on which prepare is called.
         """
         self._generate_symbols()
         self._generate_equations()
@@ -116,11 +175,15 @@ class System(object):
 
         This function is to be called after adding all device data.
         """
+        self.collect_ref()
+        self._list2array()     # `list2array` must come before `link_ext_param`
+        self.link_ext_param()
+        self.find_devices()    # find or add required devices
+        self.calc_pu_coeff()
+
+        # assign address at the end before adding devices and processing parameters
         self.set_address()
         self.set_dae_names()
-        self._collect_ref_param()
-        self._list2array()
-        self.calc_pu_coeff()
         self.store_sparse_pattern()
         self.store_adder_setter()
 
@@ -136,7 +199,7 @@ class System(object):
             logger.error('Reset failed because TDS is initialized. \nPlease reload the test case to start over.')
             return
         self.dae.reset()
-        self._call_models_method('a_reset', models=self.models)
+        self.call_model('a_reset', models=self.models)
         self.e_clear()
         self._p_restore()
         self.setup()
@@ -163,6 +226,19 @@ class System(object):
         self.__dict__[model].add(idx=idx, **param_dict)
         group.add(idx=idx, model=self.__dict__[model])
 
+        return idx
+
+    def find_devices(self):
+        """
+        Add dependent devices for all model based on `DeviceFinder`.
+        """
+        for mdl in self.models.values():
+            if len(mdl.services_fnd) == 0:
+                continue
+
+            for fnd in mdl.services_fnd.values():
+                fnd.find_or_add(self)
+
     def set_address(self, models=None):
         """
         Set addresses for differential and algebraic variables.
@@ -253,11 +329,15 @@ class System(object):
                             self.dae.z_tex_name.append(rf'${tex_name}\ {mdl_name}\ {uid}$')
                             self.dae.o += 1
 
-    def initialize(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def init(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
-        Initialize the variables in the model.
+        Initialize the variables for each of the specified models.
 
-        For each model in sequence, initializes external services and internal variables.
+        For each model, the initialization procedure is:
+
+        - Get values for all `ExtService`.
+        - Call the model `init()` method, which initializes internal variables.
+        - Copy variables to DAE and then back to the model.
         """
         if models is None:
             models = self._models_flag['pflow']
@@ -274,12 +354,15 @@ class System(object):
                 instance.link_external(ext_model)
 
             # initialize variables second
-            mdl.initialize()
+            mdl.init()
 
             # TODO: re-think over the adder-setter approach and reduce data copy
             self.vars_to_dae()
             self.vars_to_models()
 
+        # store the inverse of time constants
+        self._store_Tf()
+
     def store_adder_setter(self, models=None):
         """
         Store the adders and setters for variables and equations.
@@ -301,18 +384,25 @@ class System(object):
                 else:
                     self._setters[var.v_code].append(var)
             for item in mdl.discrete.values():
-                if isinstance(item, AntiWindupLimiter):
+                if isinstance(item, AntiWindup):
                     self.antiwindups.append(item)
 
-    def link_ext_param(self, model):
-        for instance in model.params_ext.values():
-            ext_name = instance.model
-            ext_model = self.__dict__[ext_name]
+    def link_ext_param(self, model=None):
+        if model is None:
+            models = self.models
+        else:
+            models = self._get_models(model)
 
-            try:
-                instance.link_external(ext_model)
-            except IndexError:
-                raise IndexError(f'Model <{model.class_name}> param <{instance.name}> link parameter error')
+        for model in models.values():
+            # get external parameters with `link_external` and then calculate the pu coeff
+            for instance in model.params_ext.values():
+                ext_name = instance.model
+                ext_model = self.__dict__[ext_name]
+
+                try:
+                    instance.link_external(ext_model)
+                except IndexError:
+                    raise IndexError(f'Model {model.class_name}.{instance.name} link parameter error')
 
     def calc_pu_coeff(self):
         """
@@ -324,8 +414,8 @@ class System(object):
         Sb = self.config.mva
 
         for mdl in self.models.values():
-            # get external parameters with `link_external` and then calculate the pu coeff
-            self.link_ext_param(mdl)
+            # before this step, `link_ext_param` has been called in `setup`.
+            self.link_ext_param({mdl.class_name: mdl})
 
             # default Sn to Sb if not provided. Some controllers might not have Sn or Vn.
             if 'Sn' in mdl.params:
@@ -378,19 +468,20 @@ class System(object):
 
     def l_update_var(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
-        Update variabe-based discrete components.
+        Update variable-based limiter discrete states.
 
-        This function is evaluated before equation updates.
+        This function is usually called before any equation evaluation.
         """
-        self._call_models_method('l_update_var', models)
+        self.call_model('l_update_var', models, self.dae.t)
 
     def l_check_eq(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
-        Update equation-dependent discrete components.
+        Update equation-dependent limiter discrete components.
 
-        This function is evaluated after equation updates.
+        This function is usually called after differential equation updates.
+        Currently, it is used exclusively for collecting anti-windup limiter status.
         """
-        self._call_models_method('l_check_eq', models)
+        self.call_model('l_check_eq', models)
 
     def l_set_eq(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
@@ -399,7 +490,7 @@ class System(object):
         This function is evaluated afte ``l_check_eq``.
         Currently, it is only used by anti-windup limiters to record changes.
         """
-        self._call_models_method('l_set_eq', models)
+        self.call_model('l_set_eq', models)
 
     def fg_to_dae(self):
         """
@@ -419,34 +510,46 @@ class System(object):
 
     def f_update(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
-        Call the differential equation update method for each model.
+        Call the differential equation update method for models in sequence.
 
         Notes
         -----
-        Updated equation values are not collected into DAE after this step.
+        Updated equation values remain in models and have not been collected into DAE at the end of this step.
         """
-        self._call_models_method('f_update', models)
+        try:
+            self.call_model('f_update', models)
+        except TypeError as e:
+            logger.error("f_update failed. Did you forget to run `andes prepare -q` after updating?")
+            raise e
 
     def g_update(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
-        Call the algebraic equation update method for each model.
+        Call the algebraic equation update method for models in sequence.
 
         Notes
         -----
-        Updated equation values are not collected into DAE after this step.
+        Like `f_update`, updated values have not collected into DAE at the end of the step.
         """
-        self._call_models_method('g_update', models)
+        try:
+            self.call_model('g_update', models)
+        except TypeError as e:
+            logger.error("g_update failed. Did you forget to run `andes prepare -q` after updating?")
+            raise e
 
     def j_update(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
-        Call the Jacobian update method for each model.
+        Call the Jacobian update method for models in sequence.
+
+        The procedure is
+        - Restore the sparsity pattern with :py:func:`andes.variables.dae.DAE.restore_sparse`
+        - For each sparse matrix in (fx, fy, gx, gy), evaluate the Jacobian function calls and add values.
 
         Notes
         -----
-        Updated Jacobians are reflected in the numerical DAE.
+        Updated Jacobians are immediately reflected in the DAE sparse matrices (fx, fy, gx, gy).
         """
         models = self._get_models(models)
-        self._call_models_method('j_update', models)
+        self.call_model('j_update', models)
 
         self.dae.restore_sparse()
         # collect sparse values into sparse structures
@@ -479,7 +582,7 @@ class System(object):
         term in the equations.
         """
         models = self._get_models(models)
-        self._call_models_method('store_sparse_pattern', models)
+        self.call_model('store_sparse_pattern', models)
 
         # add variable jacobian values
         for jname in jac_names:
@@ -511,11 +614,14 @@ class System(object):
 
     def vars_to_dae(self):
         """
-        From variables to dae variables
+        Copy variables values from models to `System.dae`.
+
+        This function clears `DAE.x` and `DAE.y` and collects values from models.
 
         Warnings
         --------
-        For adders, only those with `v_str` can set the value.
+        Variables with property `v_setter=False` will be added to variable only if `v_str` is not None.
+        It prevents read-intended :py:mod:`andes.core.var.ExtVar` values to be summed.
         """
         self.dae.clear_xy()
         self._v_to_dae('x')
@@ -523,7 +629,7 @@ class System(object):
 
     def vars_to_models(self):
         """
-        Send DAE variable values to models
+        Copy variable values from `System.dae` to models.
         """
         for var in self._adders['y'] + self._setters['y']:
             if var.n > 0:
@@ -566,9 +672,9 @@ class System(object):
             if var.n > 0:
                 np.put(self.dae.__dict__[v_name], var.a, var.v)
 
-    def _e_to_dae(self, eq_name):
+    def _e_to_dae(self, eq_name: str):
         """
-        Helper function for collecting equation values into dae structures `f` and `g`
+        Helper function for collecting equation values into `System.dae.f` and `System.dae.g`.
 
         Parameters
         ----------
@@ -632,9 +738,14 @@ class System(object):
 
     def dill(self):
         """
-        Serialize generated functions in ``System.calls`` with dill.
+        Serialize generated numerical functions in `System.calls` with package `dill`.
+
+        The serialized file will be stored to ``~/andes/calls.pkl``, where `~` is the home directory path.
+
+        Notes
+        -----
+        This function sets `dill.settings['recurse'] = True` to serialize the function calls recursively.
 
-        The serialized file will be stored to ``~/calls.pkl``.
         """
         logger.debug("Dumping calls to calls.pkl with dill")
         import dill
@@ -646,7 +757,10 @@ class System(object):
 
     def undill(self):
         """
-        Deserialize the function calls from `calls.pkl` with dill.
+        Deserialize the function calls from ``~/andes.calls.pkl`` with dill.
+
+        If no change is made to models, future calls to ``prepare()`` can be replaced with ``undill()`` for
+        acceleration.
         """
         import dill
         dill.settings['recurse'] = True
@@ -657,7 +771,7 @@ class System(object):
 
         with open(pkl_path, 'rb') as f:
             self.calls = dill.load(f)
-        logger.debug(f'System undill: loaded <{pkl_path}> file.')
+        logger.debug(f'Undill loaded "{pkl_path}" file.')
 
         for name, model_call in self.calls.items():
             if name in self.__dict__:
@@ -674,7 +788,7 @@ class System(object):
         if isinstance(models, str):
             models = {models: self.__dict__[models]}
         elif isinstance(models, Model):
-            models = {models.class_name, models}
+            models = {models.class_name: models}
         elif isinstance(models, list):
             models = OrderedDict()
             for item in models:
@@ -687,19 +801,51 @@ class System(object):
         # do nothing for OrderedDict type
         return models
 
-    def _call_models_method(self, method: str, models: Optional[Union[str, list, Model, OrderedDict]]):
+    def _store_Tf(self):
         """
-        Wrapper to call model methods.
+        Store the inverse time constant associated with equations
         """
-        if not isinstance(models, OrderedDict):
-            models = self._get_models(models)
-        for mdl in models.values():
-            getattr(mdl, method)()
+        for var in self._adders['f']:
+            if var.t_const is not None:
+                np.put(self.dae.Tf, var.a, var.t_const.v)
+        for var in self._setters['f']:
+            if var.t_const is not None:
+                np.put(self.dae.Tf, var.a, var.t_const.v)
+
+    def call_model(self, method: str, models: Optional[Union[str, list, Model, OrderedDict]], *args, **kwargs):
+        """
+        Call methods on the given models.
+
+        Parameters
+        ----------
+        method : str
+            Name of the model method to be called
+        models : OrderedDict, list, str
+            Models on which the method will be called
+        args
+            Positional arguments to be passed to the model method
+        kwargs
+            Keyword arguments to be passed to the model method
+
+        Returns
+        -------
+        The return value of the models in an OrderedDict
+
+        """
+        models = self._get_models(models)
+
+        ret = OrderedDict()
+        for name, mdl in models.items():
+            ret[name] = getattr(mdl, method)(*args, **kwargs)
+
+        return ret
 
     def _check_group_common(self):
         """
         Check if all group common variables and parameters are met.
 
+        This function is called at the end of code generation by `prepare`.
+
         Raises
         ------
         KeyError if any parameter or value is not provided
@@ -717,12 +863,12 @@ class System(object):
                         raise KeyError(f'Group <{group.class_name}> common param <{item}> does not exist '
                                        f'in model <{model.class_name}>')
 
-    def _collect_ref_param(self):
+    def collect_ref(self):
         """
-        Collect indices into `RefParam` for all models.
+        Collect indices into `BackRef` for all models.
         """
         for model in self.models.values():
-            for ref in model.ref_params.values():
+            for ref in model.services_ref.values():
                 ref.v = [list() for _ in range(model.n)]
 
         for model in self.models.values():
@@ -738,39 +884,42 @@ class System(object):
                     continue
 
                 for n in (model.class_name, model.group):
-                    if n not in dest_model.ref_params:
+                    if n not in dest_model.services_ref:
                         continue
 
-                    for model_idx, dest_idx in zip(model.idx, ref.v):
-                        if dest_idx not in dest_model.idx:
+                    for model_idx, dest_idx in zip(model.idx.v, ref.v):
+                        if dest_idx not in dest_model.idx.v:
                             continue
                         uid = dest_model.idx2uid(dest_idx)
-                        dest_model.ref_params[n].v[uid].append(model_idx)
+                        dest_model.services_ref[n].v[uid].append(model_idx)
 
     def _generate_pycode_file(self):
         """
         Generate empty files for storing lambdified Python code (TODO)
         """
-        self._call_models_method('generate_pycode_file', self.models)
+        self.call_model('generate_pycode_file', self.models)
 
     def _generate_initializers(self):
-        self._call_models_method('generate_initializers', self.models)
+        self.call_model('generate_initializers', self.models)
 
     def _generate_symbols(self):
-        self._call_models_method('generate_symbols', self.models)
+        self.call_model('generate_symbols', self.models)
 
     def _generate_pretty_print(self):
-        self._call_models_method('generate_pretty_print', self.models)
+        self.call_model('generate_pretty_print', self.models)
 
     def _generate_equations(self):
-        self._call_models_method('generate_equations', self.models)
+        self.call_model('generate_equations', self.models)
 
     def _generate_jacobians(self):
-        self._call_models_method('generate_jacobians', self.models)
+        self.call_model('generate_jacobians', self.models)
 
-    def _group_import(self):
+    def import_groups(self):
         """
-        Import groups defined in `devices/group.py`.
+        Import all groups classes defined in ``devices/group.py``.
+
+        Groups will be stored as instances with the name as class names.
+        All groups will be stored to dictionary ``System.groups``.
         """
         module = importlib.import_module('andes.models.group')
         for m in inspect.getmembers(module, inspect.isclass):
@@ -781,12 +930,21 @@ class System(object):
             self.__dict__[name] = cls()
             self.groups[name] = self.__dict__[name]
 
-    def _model_import(self):
+    def import_models(self):
         """
-        Import and instantiate the non-JIT models and the JIT models.
+        Import and instantiate models as System member attributes.
 
-        Models defined in ``jits`` and ``non_jits`` in ``models/__init__.py``
-        will be imported and instantiated accordingly.
+        Models defined in ``models/__init__.py`` will be instantiated `sequentially` as attributes with the same
+        name as the class name.
+        In addition, all models will be stored in dictionary ``System.models`` with model names as
+        keys and the corresponding instances as values.
+
+        Examples
+        --------
+        ``system.Bus`` stores the `Bus` object, and ``system.GENCLS`` stores the classical
+        generator object,
+
+        ``system.models['Bus']`` points the same instance as ``system.Bus``.
         """
         # non-JIT models
         for file, cls_list in non_jit.items():
@@ -795,6 +953,7 @@ class System(object):
                 the_class = getattr(the_module, model_name)
                 self.__dict__[model_name] = the_class(system=self, config=self._config_object)
                 self.models[model_name] = self.__dict__[model_name]
+                self.models[model_name].config.check()
 
                 # link to the group
                 group_name = self.__dict__[model_name].group
@@ -807,20 +966,17 @@ class System(object):
         #         self.__dict__[name] = JIT(self, file, cls, name)
         # ***********************
 
-    def _routine_import(self):
+    def import_routines(self):
         """
-        Dynamically import routines as defined in ``routines/__init__.py``.
+        Import routines as defined in ``routines/__init__.py``.
 
-        # *** DOCS FROM OLDER VERSION ***
-        The command-line argument ``--routine`` is defined in ``__cli__`` in
-        each routine file. A routine instance will be stored in the system
-        instance with the name being all lower case.
+        Routines will be stored as instances with the name as class names.
+        All groups will be stored to dictionary ``System.groups``.
 
-        For example, a routine for power flow study should be defined in
-        ``routines/pflow.py`` where ``__cli__ = 'pflow'``. The class name
-        for the routine should be ``Pflow``. The routine instance will be
-        saved to ``PowerSystem.pflow``.
-        # *******************************
+        Examples
+        --------
+        ``System.PFlow`` is the power flow routine instance, and ``System.TDS`` and ``System.EIG`` are
+        time-domain analysis and eigenvalue analysis routines, respectively.
         """
         for file, cls_list in all_routines.items():
             for cls_name in cls_list:
@@ -870,10 +1026,12 @@ class System(object):
 
     def e_clear(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
-        Clear DAE equation arrays.
+        Clear equation arrays in DAE and model variables.
+
+        This step must be called before calling `f_update` or `g_update` to flush existing values.
         """
         self.dae.clear_fg()
-        self._call_models_method('e_clear', models)
+        self.call_model('e_clear', models)
 
     def remove_pycapsule(self):
         """
@@ -891,7 +1049,7 @@ class System(object):
             self.calls[name] = mdl.calls
 
     def _list2array(self):
-        self._call_models_method('list2array', self.models)
+        self.call_model('list2array', self.models)
 
     def set_config(self, config=None):
         """
@@ -907,12 +1065,12 @@ class System(object):
 
     def get_config(self):
         """
-        Get config data from models
+        Collect config data from models.
 
         Returns
         -------
         dict
-            a dict containing the config from devices; class names are the keys
+            a dict containing the config from devices; class names are keys and configs in a dict are values.
         """
         config_dict = configparser.ConfigParser()
         config_dict[self.__class__.__name__] = self.config.as_dict()
@@ -929,12 +1087,12 @@ class System(object):
     @staticmethod
     def load_config(conf_path=None):
         """
-        Load config from an ``andes.rc`` file.
+        Load config from an rc-formatted file.
 
         Parameters
         ----------
         conf_path : None or str
-            Path to the Andes rc file. If ``None``, the function body
+            Path to the config file. If is `None`, the function body
             will not run.
 
         Returns
@@ -946,18 +1104,25 @@ class System(object):
 
         conf = configparser.ConfigParser()
         conf.read(conf_path)
-        logger.debug(f'Config: Loaded from file <{conf_path}>.')
+        logger.debug(f'Config loaded from file "{conf_path}".')
         return conf
 
-    def save_config(self, file_path=None):
+    def save_config(self, file_path=None, overwrite=False):
         """
-        Save system and routine configurations to an rc-formatted file.
+        Save all system, model, and routine configurations to an rc-formatted file.
 
         Parameters
         ----------
-        file_path : str
-            path to the configuration file. The user will be prompted if the
-            file already exists.
+        file_path : str, optional
+            path to the configuration file default to `~/andes/andes.rc`.
+        overwrite : bool, optional
+            If file exists, True to overwrite without confirmation.
+            Otherwise prompt for confirmation.
+
+        Warnings
+        --------
+        Saved config is loaded back and populated *at system instance creation time*.
+        Configs from the config file takes precedence over default config values.
         """
         if file_path is None:
             andes_path = os.path.join(os.path.expanduser('~'), '.andes')
@@ -965,9 +1130,7 @@ class System(object):
             file_path = os.path.join(andes_path, 'andes.rc')
 
         elif os.path.isfile(file_path):
-            choice = input(f'Config file {file_path} already exist. Overwrite? [y/N]').lower()
-            if len(choice) == 0 or choice[0] != 'y':
-                logger.info('No config file overwritten.')
+            if not confirm_overwrite(file_path, overwrite=overwrite):
                 return
 
         conf = self.get_config()
@@ -975,6 +1138,7 @@ class System(object):
             conf.write(f)
 
         logger.info(f'Config written to "{file_path}"')
+        return file_path
 
     def supported_models(self):
         """
